package com.ozs.service.service.impl;

import com.ozs.common.utils.AppendUtils;
import com.ozs.service.entity.BaseDeviceDynamicManagement;
import com.ozs.service.entity.CameraTree;
import com.ozs.service.entity.vo.BaseCameraManagementDto;
import com.ozs.service.entity.vo.CameraTreeVo;
import com.ozs.service.mapper.BaseDeviceDynamicManagementMapper;
import com.ozs.service.service.BaseDeviceDynamicManagementService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备动态管理表 服务实现类
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@Service
public class BaseDeviceDynamicManagementServiceImpl extends ServiceImpl<BaseDeviceDynamicManagementMapper, BaseDeviceDynamicManagement> implements BaseDeviceDynamicManagementService {
    @Autowired
    BaseDeviceDynamicManagementMapper baseDeviceDynamicManagementMapper;

    @Override
    public CameraTree cameraTree(CameraTreeVo cameraTreeVo) {
        //1 父
        CameraTree cameraTree = CameraTree.builder()
                .name("相机树")
                .code("0")
                .flay(false)
                .children(new ArrayList<>())
                .build();
        Map<String, CameraTree> map = new HashMap<>();
        List<BaseCameraManagementDto> list = baseDeviceDynamicManagementMapper.list(cameraTreeVo);
        if (list.size() > 0) {
            for (BaseCameraManagementDto o : list) {
                //行别字符串
                String lineDirName = "";
                if ("1".equals(o.getLineDir())) {
                    lineDirName = "上行";
                } else if ("2".equals(o.getLineDir())) {
                    lineDirName = "下行";
                } else if ("3".equals(o.getLineDir())) {
                    lineDirName = "上下行";
                }

                // 线路
                CameraTree.CameraTreeBuilder builder = CameraTree.builder()
                        .code(o.getRailwayCode())
                        .railwayName(o.getRailwayName())
                        .flay(true)
                        .publicWorksSection(o.getPublicWorksSection())
                        .children(new ArrayList<>());
                if (o.getLineNumber() != null && !o.getLineNumber().isEmpty()) {
                    builder.number(Integer.valueOf(o.getLineNumber()));
                    builder.name(o.getLineNumber()+o.getRailwayName());
                }else {
                    builder.name(o.getRailwayName());
                }
                CameraTree build = builder.build();
                if (!cameraTree.getChildren().stream()
                        .map(CameraTree::getCode)
                        .collect(Collectors.toList())
                        .contains(build.getCode())) {
                    cameraTree.getChildren().add(build);
                    map.put(o.getRailwayCode(), build);
                }
                //工务段
                CameraTree build3 = CameraTree.builder()
                        .code(o.getRailwayCode())
                        .name(o.getLineNumber() + o.getRailwayName())
                        .railwayName(o.getRailwayName())
                        .flay(true)
                        .publicWorksSection(o.getPublicWorksSection())
                        .children(new ArrayList<>())
                        .build();
                if (!ObjectUtils.isEmpty(map.get(o.getRailwayCode()))) {
                    CameraTree cameraTree3 = map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection());
                    if (ObjectUtils.isEmpty(cameraTree3)) {
                        map.put(o.getRailwayCode() + "-" + o.getPublicWorksSection(), build3);
                        map.get(o.getRailwayCode()).getChildren().add(build3);
                    }
                }

                // 里程
                String mils = AppendUtils.stringAppend(o.getInstallMile());
                CameraTree build1 = CameraTree.builder()
                        .code(mils + "-" + lineDirName)
                        .name(mils + "-" + lineDirName)
                        .flay(true)
                        .publicWorksSection(o.getPublicWorksSection())
                        .mileage(o.getInstallMile())
                        .children(new ArrayList<>())
                        .build();
                if (!ObjectUtils.isEmpty(map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection()))) {
                    CameraTree cameraTree1 = map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection() + "-" + o.getInstallMile() + "-" + lineDirName);
                    if (ObjectUtils.isEmpty(cameraTree1)) {
                        map.put(o.getRailwayCode() + "-" + o.getPublicWorksSection() + "-" + o.getInstallMile() + "-" + lineDirName, build1);
                        map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection()).getChildren().add(build1);
                    }
                }
                // 摄像头
                CameraTree build2 = CameraTree.builder()
                        .code(o.getCameraCode())
                        .name(o.getCameraCode())
                        .flay(true)
                        .publicWorksSection(o.getPublicWorksSection())
                        .children(new ArrayList<>())
                        .build();
                if (!ObjectUtils.isEmpty(map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection() + "-" + o.getInstallMile() + "-" + lineDirName))) {
                    map.get(o.getRailwayCode() + "-" + o.getPublicWorksSection() + "-" + o.getInstallMile() + "-" + lineDirName).getChildren().add(build2);
                }
            }
        }
        return cameraTree;
    }
}
