package com.ozs.service.service.impl;

import com.ozs.common.utils.MinioUtils;
import com.ozs.common.utils.uuid.IdUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.util.Date;

/**
 * @author wyy
 * @subject
 * @creat 2023/4/26
 */
@Service
public class CameraCaptureService {
    private static final Logger log = LoggerFactory.getLogger(CameraCaptureService.class);
    @Resource
    MinioUtils minioUtils;


    private final static Integer CONST_TIME_OUT = 60 * 1000;

    public boolean getCapture(URL url, String fileName) throws IOException {
        return getCapture(url, "", fileName);
    }

    public boolean getCapture(URL url, String cameraCode, String fileName) throws IOException {
        HttpURLConnection connection = null;
//        BufferedInputStream bis = null;
//        BufferedOutputStream bos = null;
        ByteArrayOutputStream byteArrayOutputStream = null;
        ByteArrayInputStream byteArrayInputStream = null;
        FileOutputStream fileOutputStream = null;
        try {
            connection = (HttpURLConnection) url.openConnection();
            // http正文内，因此需要设为true
            connection.setDoOutput(true);
            connection.setDoInput(true);
            // Post 请求不能使用缓存
            connection.setUseCaches(false);
            //设置本次连接是否自动重定向
            connection.setInstanceFollowRedirects(true);

            connection.setRequestMethod("GET");
            connection.setRequestProperty("Accept-Language", "zh,zh-CN;q=0.9");
            connection.setRequestProperty("connection", "keep-alive");
            //设置参数类型是json格式
            connection.setRequestProperty("Content-Type", "application/json;charset=utf-8");
            connection.setRequestProperty("Accept","application/json");
            //connection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
            connection.setRequestProperty("Charsert", "UTF-8");
            //设置读取时间为30秒
            connection.setConnectTimeout(CONST_TIME_OUT);
            connection.setReadTimeout(CONST_TIME_OUT);
            connection.connect();

            // 返回流
//            System.out.println("responseCode: " + connection.getResponseCode()+", url: "+url.toString());
//            log.info("responseCode: " + connection.getResponseCode()+", url: " + url);
            if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
                InputStream input = connection.getInputStream();
//                bis = new BufferedInputStream(input);
                byte [] bytes = new byte[102400];
                int index = 0;
                byteArrayOutputStream = new ByteArrayOutputStream();
                while (-1 != (index = input.read(bytes,0,bytes.length))){
                    byteArrayOutputStream.write(bytes,0,index);
                }
                byteArrayInputStream = new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
                //String uploadUrl = "/data/test_picture/"+ IdUtils.fastSimpleUUID()+".jpeg";
//                String uploadUrl = "d://tmp/c/"+ (cameraCode + "-" + (System.currentTimeMillis() / 1000)) +".jpeg";
//                if (log.isDebugEnabled()) {
//                    log.debug("生成至服务器本地开始，路径为: " + uploadUrl);
//                }
                //fileOutputStream = new FileOutputStream(uploadUrl);
                //IOUtils.copy(byteArrayInputStream, fileOutputStream);
//                if (log.isDebugEnabled()) {
//                    log.debug("生成至服务器本地结束，路径为: " + uploadUrl);
//                }
                //byteArrayInputStream.reset();
//                if (log.isDebugEnabled()) {
//                    log.debug("getCapture======" + fileName);
//                }
                minioUtils.minIoClientUpload(byteArrayInputStream, fileName);
//                if (log.isDebugEnabled()) {
//                    log.debug("fileName: " + fileName + ", 图片结束截图时间: " + new Date());
//                }
                return true;
            } else {
                if (log.isDebugEnabled()) {
                    log.debug("请求摄像头截图接口出错, url: " + url + ", 反馈错误状态码: " + connection.getResponseCode() +
                            ", 错误内容: " + connection.getResponseMessage());
                }
            }
        } catch (Exception e) {
            //先关闭外层的缓冲流，再关闭内层的流,但是在关闭外层流的同时，
            //内层流也会自动的进行关闭，关于内层流的关闭，可以省略
//            e.printStackTrace();
            log.error("图片截图出现异常情形: " + e.getMessage());
        } finally {
//            if (bis != null) {
//                bis.close();
//            }
            if (fileOutputStream != null) {
                fileOutputStream.close();
            }
            if (byteArrayInputStream != null) {
                byteArrayInputStream.close();
            }
            if (connection != null) {
                connection.disconnect();
            }
            if(byteArrayOutputStream != null){
                byteArrayOutputStream.flush();
                byteArrayOutputStream.close();
            }
        }
        return false;
    }
}
