package com.ozs.service.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageHelper;
import com.ozs.common.constant.Constants;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.vo.*;
import com.ozs.service.mapper.BaseCameraManagementMapper;
import com.ozs.service.mapper.BaseRailwayManagementMapper;
import com.ozs.service.mapper.MsgAlarmMapper;
import com.ozs.service.service.MsgAlarmService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ozs.service.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 报警信息表 服务实现类
 * </p>
 *
 * @author ozs
 * @since 2023-02-22
 */
@Service
@Slf4j
public class MsgAlarmServiceImpl extends ServiceImpl<MsgAlarmMapper, MsgAlarm> implements MsgAlarmService {
    @Autowired
    private MsgAlarmMapper msgAlarmMapper;
    @Autowired
    private BaseCameraManagementMapper baseCameraManagementMapper;
    @Autowired
    private RedisService redisService;
    @Autowired
    private BaseRailwayManagementMapper baseRailwayManagementMapper;


    public IPage listToPage(List list, int pageNum, int pageSize) {
        List pageList = new ArrayList<>();
        int curIdx = pageNum > 1 ? (pageNum - 1) * pageSize : 0;
        for (int i = 0; i < pageSize && curIdx + i < list.size(); i++) {
            pageList.add(list.get(curIdx + i));
        }
        IPage page = new Page<>(pageNum, pageSize);
        page.setRecords(pageList);
        page.setTotal(list.size());
        return page;
    }

    //web数据统计 报警数据详情
    @Override
    public IPage<AlarmStatisticVo> list(AlarmStatisticResVo alarmStatisticResVo) {
        int pageNum = Integer.parseInt(alarmStatisticResVo.getPageNum().toString());
        int pageSize = Integer.parseInt(alarmStatisticResVo.getPageSize().toString());
        com.github.pagehelper.Page<AlarmStatisticVo> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.list(alarmStatisticResVo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<AlarmStatisticVo> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public List<AlarmStatisticMonthDto> getCount(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticMonthDto> list = msgAlarmMapper.listDtoMonth(alarmStatisticResVo);
        return list;
    }

    @Override
    public List<AlarmStatisticMonthDto> getDayCount(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticMonthDto> list = msgAlarmMapper.listDtoDay(alarmStatisticResVo);
        return list;
    }

    @Override
    public List<AlarmStatisticDto> exportExcel(AlarmStatisticResVo alarmStatisticResVo) {
        List<AlarmStatisticDto> listDto = msgAlarmMapper.listDto(alarmStatisticResVo);
        return listDto;
    }

    @Override
    public MsgAlarm selectByCameraCode(String cameraCode) {
        return msgAlarmMapper.selectByCameraCode(cameraCode);
    }

    @Override
    public IPage<MsgAlarm> queryPage(MsgAlarmVo vo) {
        int pageNum = Integer.parseInt(vo.getPageNum().toString());
        int pageSize = Integer.parseInt(vo.getPageSize().toString());
        com.github.pagehelper.Page<MsgAlarm> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.selectMsgAlarmList(vo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<MsgAlarm> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public MsgAlarm getByAlarmId(String alarmId) {
        return msgAlarmMapper.getByAlarmId(alarmId);
    }

    @Override
    public Map<String, Integer> statisticsNum(MsgAlarmVo msgAlarmVo) {
        Map<String, Integer> map = new HashMap<>();
        // 相机数量
        Integer countCamera = baseCameraManagementMapper.countCamera(msgAlarmVo);
        // 累计报警次数
        Integer countMsg = msgAlarmMapper.countMsg(msgAlarmVo);
        // 解除报警
//        LambdaQueryWrapper<MsgAlarm> lwJC = new LambdaQueryWrapper<MsgAlarm>();
//        lwJC.eq(MsgAlarm::getIsLock, "1");
        Integer countJC = msgAlarmMapper.countJC(msgAlarmVo);
        // 未解除报警
//        LambdaQueryWrapper<MsgAlarm> lwWJC = new LambdaQueryWrapper<MsgAlarm>();
//        lwWJC.eq(MsgAlarm::getIsLock, "2");
        Integer countWJC = msgAlarmMapper.countWJC(msgAlarmVo);
        map.put("countCamera", countCamera);
        map.put("countMsg", countMsg);
        map.put("countJC", countJC);
        map.put("countWJC", countWJC);
        return map;
    }

    @Override
    public Integer appAlarmNum(MsgAlarmVo msgAlarmVo) {
        return msgAlarmMapper.countWJC(msgAlarmVo);
    }

    @Override
    public List<MsgAlarm> msgAlarmList(MsgAlarmVo msgAlarmVo) {
        return msgAlarmMapper.selectMsgAlarmList(msgAlarmVo);
    }

    @Override
    public List<MsgAlarm> queryAlarmInfoByAfterTenMinute() {
        return msgAlarmMapper.queryAlarmInfoByAfterTenMinute();
    }

    @Override
    public IPage<MsgAlarm> queryPageHome(MsgAlarmVo msgAlarmVo) {
        int pageNum = Integer.parseInt(msgAlarmVo.getPageNum().toString());
        int pageSize = Integer.parseInt(msgAlarmVo.getPageSize().toString());
        com.github.pagehelper.Page<MsgAlarm> page = PageHelper.startPage(pageNum, pageSize)
                .doSelectPage(() -> msgAlarmMapper.selectMsgAlarmListHome(msgAlarmVo));
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<MsgAlarm> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(pageNum, pageSize);
        pageR.setRecords(page.getResult());
        pageR.setTotal(page.getTotal());
        return pageR;
    }

    @Override
    public void getNormalBaseCameraManagementsCache(String setType) {
        log.info("-----------setType------" + setType);
        Set<BaseCameraVO> alarmSet = redisService.getSetMembers(Constants.ALARM_SET);
        log.info("-----------alarmSet------" + alarmSet);
        if (!ObjectUtils.isEmpty(alarmSet)) {
            Set<String> cameraCodeSet = new HashSet<>();
            alarmSet.forEach(camera -> {
                if (!ObjectUtils.isEmpty(camera)) {
                    if (!StringUtils.isEmpty(camera.getCameraCode())) {
                        cameraCodeSet.add(camera.getCameraCode());
                    }
                }
            });
            log.info("-----------cameraCodeSet------" + cameraCodeSet);
            QueryWrapper<BaseCameraManagement> queryWrapper = new QueryWrapper<>();
            log.info("-----------Constants.ALARM_SET------" + Constants.ALARM_SET);
            if (Constants.ALARM_SET.equals(setType)) {
                // 使用QueryWrapper和notin方法根据cameraCode列表查询BaseCameraManagement数据
                queryWrapper.notIn("camera_code", cameraCodeSet);
                Set<BaseCameraManagement> set = baseCameraManagementMapper.selectList(queryWrapper).stream().collect(Collectors.toSet());
                log.info("-----------set------" + set);
                set.forEach(item -> {
                    BaseCameraVO baseCameraVO = new BaseCameraVO();
                    baseCameraVO.setCameraCode(item.getCameraCode());
                    baseCameraVO.setChannel(item.getChannel());
                    redisService.addToSet(Constants.NORMAL_SET, baseCameraVO);
                });
            }
        }
    }

    @Override
    public void initializationCameraCache() {
        List<MsgAlarm> lockedMsgAlarms = msgAlarmMapper.selectList(new QueryWrapper<MsgAlarm>().eq("is_lock", 2));
        if (lockedMsgAlarms.isEmpty()) {
            Set<BaseCameraManagement> normalSet = baseCameraManagementMapper.selectList(null).stream().collect(Collectors.toSet());
            normalSet.forEach(item -> {
                BaseCameraVO baseCameraVO = new BaseCameraVO();
                baseCameraVO.setCameraCode(item.getCameraCode());
                baseCameraVO.setChannel(item.getChannel());
                redisService.addToSet(Constants.NORMAL_SET, baseCameraVO);
            });
            redisService.addToSet(Constants.ALARM_SET, null);
        }
        // 提取cameraCode列表
        Set<String> cameraCodeSet = lockedMsgAlarms.stream().map(MsgAlarm::getCameraCode).collect(Collectors.toSet());
        QueryWrapper<BaseCameraManagement> queryWrapperAlarm = new QueryWrapper<>();
        queryWrapperAlarm.in("camera_code", cameraCodeSet);
        Set<BaseCameraManagement> alarmSet = baseCameraManagementMapper.selectList(queryWrapperAlarm).stream().collect(Collectors.toSet());
        alarmSet.forEach(item -> {
            BaseCameraVO baseCameraVO = new BaseCameraVO();
            baseCameraVO.setCameraCode(item.getCameraCode());
            baseCameraVO.setChannel(item.getChannel());
            redisService.addToSet(Constants.ALARM_SET, baseCameraVO);
        });
        QueryWrapper<BaseCameraManagement> queryWrapperNormal = new QueryWrapper<>();
        queryWrapperNormal.notIn("camera_code", cameraCodeSet);
        Set<BaseCameraManagement> normalSet = baseCameraManagementMapper.selectList(queryWrapperNormal).stream().collect(Collectors.toSet());
        normalSet.forEach(item -> {
            BaseCameraVO baseCameraVO = new BaseCameraVO();
            baseCameraVO.setCameraCode(item.getCameraCode());
            baseCameraVO.setChannel(item.getChannel());
            redisService.addToSet(Constants.NORMAL_SET, baseCameraVO);
        });
    }

    @Override
    public BaseCameraManagement getBaseCameraManagementByCamereCode(String camereCode) {
        QueryWrapper<BaseCameraManagement> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("camera_code", camereCode);
        BaseCameraManagement baseCameraManagement = baseCameraManagementMapper.selectOne(queryWrapper);
        return baseCameraManagement;
    }

    @Override
    public AjaxResult alarmStatistics(MsgAlarmVo msgAlarmVo1) {
        int count = 0;
        int unresolvedCount = 0;
        //找出所有报警数据
        List<MsgAlarm> list = msgAlarmMapper.selectMsgAlarmList(msgAlarmVo1);
        //未解除的报警
        msgAlarmVo1.setIsLock(2);
        List<MsgAlarm> alarmList = msgAlarmMapper.selectMsgAlarmList(msgAlarmVo1);
        HashMap<String, Integer> map = new HashMap<>();
        if (!ObjectUtils.isEmpty(list)) {
            //获取所有报警的相机编码
            //找出所有报警数据 1=高铁的数量
            count = (int) list.stream()
                    .map(base -> baseCameraManagementMapper.selectOne(new LambdaQueryWrapper<BaseCameraManagement>()
                            .eq(BaseCameraManagement::getCameraCode, base.getCameraCode())))
                    .filter(Objects::nonNull)  // 确保 management 不是 null
                    .map(management -> baseRailwayManagementMapper.selectOne(new LambdaQueryWrapper<BaseRailwayManagement>()
                            .eq(BaseRailwayManagement::getRailwayCode, management.getRailwayCode()))) // 确保管理信息不是空的
                    .filter(Objects::nonNull) // 确保 baseRailwayManagement 不是 null
                    .filter(baseRailwayManagement -> baseRailwayManagement.getRailwayType() == 1)
                    .count();
            //普铁的数量
            int pu = list.size() - count;
            map.put("highAlarmCount", count);
            map.put("puAlarmCount", pu);
        } else {
            map.put("highAlarmCount", 0);
            map.put("puAlarmCount", 0);
        }
        if (!ObjectUtils.isEmpty(alarmList)) {
            //获取所有报警的相机编码
            //找出所有未解除报警数据 1=高铁的数量
            unresolvedCount = (int) alarmList.stream()
                    .map(base -> baseCameraManagementMapper.selectOne(new LambdaQueryWrapper<BaseCameraManagement>()
                            .eq(BaseCameraManagement::getCameraCode, base.getCameraCode())))
                    .filter(Objects::nonNull)
                    .map(management -> baseRailwayManagementMapper.selectOne(new LambdaQueryWrapper<BaseRailwayManagement>()
                            .eq(BaseRailwayManagement::getRailwayCode, management.getRailwayCode()))) // 确保管理信息不是空的
                    .filter(Objects::nonNull) // 确保 baseRailwayManagement 不是 null
                    .filter(baseRailwayManagement -> baseRailwayManagement.getRailwayType() == 1)
                    .count();
            //普铁的数量
            int puUnresolvedCount = alarmList.size() - unresolvedCount;
            map.put("highUnresolvedCount", unresolvedCount);
            map.put("puUnresolvedCount", puUnresolvedCount);
        } else {
            map.put("highUnresolvedCount", 0);
            map.put("puUnresolvedCount", 0);
        }
        return AjaxResult.success(map);
    }

    @Override
    public AjaxResult alarmCountSort(MsgAlarmVo msgAlarmVo) {
        return AjaxResult.success(baseCameraManagementMapper.alarmCountSort(msgAlarmVo));
    }
}
