package com.ozs.web.controller.accountmanagment;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ozs.common.annotation.Log;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.bean.BeanUtils;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseCameraVersion;
import com.ozs.service.entity.BaseVersionCameraParam;
import com.ozs.service.entity.vo.BaseCameraManagementHomeVo;
import com.ozs.service.entity.vo.BaseCameraVersionVo;
import com.ozs.service.service.BaseCameraManagementService;
import com.ozs.service.service.BaseCameraVersionService;
import com.ozs.service.service.BaseVersionCameraParamService;
import com.ozs.system.service.ISysDictDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 相机版本管理
 * @Author : sunhh
 * @create 2023/4/24 11:00
 */
@Slf4j
@RestController
@RequestMapping("/baseCameraVersion")
public class BaseCameraVersionController extends BaseController {

    @Autowired
    private BaseCameraVersionService baseCameraVersionService;
    @Resource
    private BaseCameraManagementService baseCameraManagementService;
    @Autowired
    private ISysDictDataService dictDataService;
    @Autowired
    private BaseVersionCameraParamService baseVersionCameraParamService;

    /**
     * 相机版本管理 版本升级
     *
     * @param baseCameraVersion
     * @return
     */
    @PostMapping("/cameraVersionUpgrade")
    @Log(title = "相机版本管理", businessType = BusinessType.INSERT)
    public AjaxResult cameraVersionUpgrade(@RequestBody BaseCameraVersionVo baseCameraVersion) {
        if (StringUtils.isEmptySunhh(baseCameraVersion) || StringUtils.isEmptySunhh(baseCameraVersion.getId())) {
            return AjaxResult.error("相机版本id不能为空！");
        }
        if (StringUtils.isEmptySunhh(baseCameraVersion) || baseCameraVersion.getCameraCodeList().size() == 0) {
            return AjaxResult.error("相机编码列表 不能为空！");
        }
        return baseCameraVersionService.cameraVersionUpgrade(baseCameraVersion, getUserId());
    }

    /**
     * 相机列表
     *
     * @param baseCameraManagement
     * @return
     */
    @PostMapping("/cameraAllList")
    @Log(title = "相机版本管理", businessType = BusinessType.SELECT)
    public AjaxResult cameraAllList(@RequestBody BaseCameraManagement baseCameraManagement) {
        if (StringUtils.isEmptySunhh(baseCameraManagement) || StringUtils.isEmptySunhh(baseCameraManagement.getVersionId())) {
            return AjaxResult.error("相机版本id不能为空！");
        }
        List<BaseCameraManagement> baseCameraManagements = baseCameraManagementService.cameraAllList(baseCameraManagement);
        return AjaxResult.success(baseCameraManagements);
    }

    /**
     * 相机版本管理 分页查询
     *
     * @param baseCameraVersionVo
     * @return
     */
    @PostMapping("/cameraVersionList")
    @Log(title = "相机版本管理", businessType = BusinessType.SELECT)
    public AjaxResult cameraVersionList(@RequestBody BaseCameraVersionVo baseCameraVersionVo) {
        com.github.pagehelper.Page<BaseCameraVersion> page = baseCameraVersionService.cameraVersionList(baseCameraVersionVo);
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<BaseCameraVersionVo> pageR =
                new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(baseCameraVersionVo.getPageNum(), baseCameraVersionVo.getPageSize());
        pageR.setTotal(page.getTotal());
        if (!ObjectUtils.isEmpty(page) && page.getResult().size() > 0) {
            List<BaseCameraVersionVo> dto1 = page.getResult().stream().map(o -> {
                BaseCameraVersionVo baseCameraVersionVo1 = new BaseCameraVersionVo();
                BeanUtils.copyProperties(o, baseCameraVersionVo1);
                // 升级类型 0.固件 1.算法 2:算法及硬件升级 3:摄像头系统参数
                String upgradeTypeName = dictDataService.selectDictLabel("upgrade_type", o.getUpgradeType());
                if (!StringUtils.isEmptySunhh(upgradeTypeName)) {
                    baseCameraVersionVo1.setUpgradeTypename(upgradeTypeName);
                }
                if ("3".equals(o.getUpgradeType())) {
                    LambdaQueryWrapper<BaseVersionCameraParam> wrapperBaseVersionCameraParam = new LambdaQueryWrapper<>();
                    if (!StringUtils.isEmptySunhh(baseCameraVersionVo) && !StringUtils.isEmptySunhh(baseCameraVersionVo.getId())) {
                        wrapperBaseVersionCameraParam.eq(BaseVersionCameraParam::getVersionId, baseCameraVersionVo.getId());
                    }
                    List<BaseVersionCameraParam> baseVersionCameraParams = baseVersionCameraParamService.list(wrapperBaseVersionCameraParam);
                    baseCameraVersionVo1.setBaseVersionCameraParamList(baseVersionCameraParams);
                }
                return baseCameraVersionVo1;
            }).collect(Collectors.toList());
            pageR.setRecords(dto1);
        }
        return AjaxResult.success(pageR);
    }

    /**
     * 相机版本管理 新增
     *
     * @param baseCameraVersion
     * @return
     */
    @PostMapping("/cameraVersionAdd")
    @Log(title = "相机版本管理", businessType = BusinessType.INSERT)
    public AjaxResult cameraVersionAdd(@RequestBody BaseCameraVersion baseCameraVersion) {
        if (StringUtils.isEmptySunhh(baseCameraVersion) || StringUtils.isEmptySunhh(baseCameraVersion.getVersionNum())
                || StringUtils.isEmptySunhh(baseCameraVersion.getVersionName())
                || StringUtils.isEmptySunhh(baseCameraVersion.getUpgradeType())) {
            return AjaxResult.error("相机版本号、版本名称、版本类型 不能为空！");
        }
        return baseCameraVersionService.cameraVersionAdd(baseCameraVersion, getUserId());
    }

    /**
     * 相机版本管理 删除
     *
     * @param ids
     * @return
     */
    @RequestMapping("/cameraVersionDelete/{ids}")
    @Log(title = "相机版本管理", businessType = BusinessType.DELETE)
    public AjaxResult cameraVersionDelete(@PathVariable List<Long> ids) {
        if (StringUtils.isEmptySunhh(ids)) {
            return AjaxResult.error("删除ID不能为空！");
        }
        for (Long id : ids) {
            LambdaQueryWrapper<BaseCameraManagement> wrapper = new LambdaQueryWrapper<>();
            if (!StringUtils.isEmptySunhh(id)) {
                wrapper.eq(BaseCameraManagement::getVersionId, id);
            }
            List<BaseCameraManagement> list = baseCameraManagementService.list(wrapper);
            if (!StringUtils.isEmptySunhh(list) && list.size() > 0) {
                return AjaxResult.error("删除ID: " + id + "，已经升级了相机，不能删除！");
            }
            boolean delete = baseCameraVersionService.removeById(id);
        }
        // boolean delete = baseCameraVersionService.removeByIds(ids);
        return AjaxResult.success();
    }
}
