package com.ozs.web.controller.app;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;

import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.utils.ImgUtil;
import com.ozs.common.utils.MinioUtils;
import com.ozs.common.utils.SecurityUtils;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.uuid.IdUtils;
import com.ozs.framework.web.service.TokenService;
import com.ozs.service.entity.BaseCameraManagement;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.MsgAlarm;
import com.ozs.service.entity.MsgAppPush;
import com.ozs.service.entity.vo.MsgAlarmVo;
import com.ozs.service.entity.vo.MsgAppPushVo;
import com.ozs.service.service.MsgAppPushService;
import com.ozs.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @author wyy
 * @subject
 * @creat 2023/3/6
 */
@RestController
@RequestMapping("/app/mine")
@Slf4j
public class GeoHazardMineController extends BaseController {
    public static final String PW_PATTERN = "^(?![A-Za-z0-9]+$)(?![a-z0-9\\W]+$)(?![A-Za-z\\W]+$)(?![A-Z0-9\\W]+$)[a-zA-Z0-9\\W]{8,20}$";
    @Autowired
    private ISysUserService userService;
    @Autowired
    private TokenService tokenService;
    @Autowired
    private ISysUserService sysUserService;
    @Autowired
    private MsgAppPushService msgAppPushService;
    @Resource
    private MinioUtils minioUtils;
    @Value("${base.imgUrl}")
    private String imgUrl;

    /**
     * 修改头像
     */
    @PostMapping("/updateAvatar")
    public AjaxResult updateAvatar(@RequestParam("image") MultipartFile image, @RequestParam("userId") String userId) {
        if (image.isEmpty()) {
            return AjaxResult.error("不能上传空文件哦");
        }
        //图片保存路径
        //String fileUploadPath ="/"+userId+"/image";
        String uploadFile = null;
        if (image != null && !image.isEmpty()) {
            String imageName = image.getOriginalFilename();
            if (StringUtils.isNotBlank(imageName)) {
                // String date = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
                String filename = IdUtils.fastSimpleUUID() + image.getOriginalFilename().substring(image.getOriginalFilename().lastIndexOf("."));
                SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy-MM-dd");
                String format1 = dateFormat1.format(new Date());
                String imgName = format1 + "/" + filename;
                try {
                    minioUtils.minIoClientUpload(image.getInputStream(), imgName);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                uploadFile = imgUrl + "/" + format1 + "/" + filename;
                sysUserService.updateUserAvatarById(userId, uploadFile);
            }
        } else {
            return AjaxResult.error("上传功能出错");
        }
        return AjaxResult.success(uploadFile);
    }

    /**
     * 修改密码
     */
    @RequestMapping(value = "/updatePassword", method = RequestMethod.POST)
    public AjaxResult updatePassword(@RequestBody SysUser user) {
//        sysUserService.updatePassword(userId, newPwd);
//        return AjaxResult.success();
        LoginUser loginUser = getLoginUser();
        String password = user.getPassword();
        String s = SecurityUtils.encryptPassword(password);
        if (!SecurityUtils.matchesPassword(user.getPassword(), loginUser.getPassword())) {
            return error("修改密码失败，旧密码错误");
        }
        boolean matches = user.getNewPassword().matches(PW_PATTERN);
        if (!matches) {
            return error("您的密码太简单!需要包含大小英文、数字、特殊字符、并且长度8-20");
        }
        if (SecurityUtils.matchesPassword(user.getNewPassword(), loginUser.getPassword())) {
            return error("新密码不能与旧密码相同");
        }
        user.setPassword(SecurityUtils.encryptPassword(user.getNewPassword()));
        user.setUpdateBy(getUsername());
        user.setId(getId());
        if (userService.resetPwd(user) > 0) {
            // 更新缓存用户密码
            loginUser.getUser().setPassword(SecurityUtils.encryptPassword(user.getNewPassword()));
            tokenService.setLoginUser(loginUser);
            return success();
        }
        return error("修改密码异常，请联系管理员");
    }

    /**
     * app推送报警信息记录分页显示（全部信息）
     */
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public AjaxResult list(@RequestBody MsgAlarmVo msgAlarmVo) {
        String userId = getUserId();
        if (!StringUtils.isEmpty(userId)) {
            msgAlarmVo.setUserId(userId);
        }
        IPage<MsgAlarm> list = msgAppPushService.alarmPushMsg(msgAlarmVo);
        return AjaxResult.success(list);
    }

    /**
     * app推送报警信息记录修改状态
     */
    @RequestMapping(value = "/update", method = RequestMethod.POST)
    public AjaxResult update(@RequestBody MsgAppPush msgAppPush) {
        msgAppPush.setUpdateBy(getUsername());
        msgAppPushService.updates(msgAppPush);
        return AjaxResult.success();
    }
}

