package com.ozs.web.controller.system;

import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.domain.vo.SysDeptVo;
import com.ozs.common.utils.poi.ExcelUtil;
import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.UserConstants;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.StringUtils;
import com.ozs.system.service.ISysDeptService;

import javax.servlet.http.HttpServletResponse;

/**
 * 部门信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/dept")
public class SysDeptController extends BaseController {
    @Autowired
    private ISysDeptService deptService;

    /**
     * 获取部门列表
     */
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list")
    public AjaxResult list(SysDeptVo dept) {
        List<SysDept> depts = deptService.selectDeptList(dept);
        for (SysDept sysDept : depts) {
            for (SysDept dept1 : depts) {
                if (dept1.getDeptId().equals(sysDept.getParentId())) {
                    sysDept.setParentName(dept1.getDeptName());
                    break;
                }
            }
        }
        return success(depts);
    }

    /**
     * 获取部门列表,
     */
    @PreAuthorize("@ss.hasPermi('system:dept:listAll')")
    @GetMapping("/listAll")
    public AjaxResult listAll(SysDept dept) {
        List<SysDept> sysDeptS = forDeptList(dept);
        return success(sysDeptS);
    }

    /**
     * 获取部门树列表
     */
    @GetMapping("/deptTree")
    public AjaxResult deptTree(SysDeptVo dept) {
        return success(deptService.selectDeptTreeList(dept));
    }

    /**
     * 查询部门列表（排除节点）
     */
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list/exclude/{deptId}")
    public AjaxResult excludeChild(@PathVariable(value = "deptId", required = false) Long deptId) {
        List<SysDept> depts = deptService.selectDeptList(new SysDeptVo());
        depts.removeIf(d -> d.getDeptId().intValue() == deptId || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + ""));
        return success(depts);
    }

    /**
     * 根据部门编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:dept:query')")
    @GetMapping(value = "/{deptId}")
    public AjaxResult getInfo(@PathVariable Long deptId) {
        //deptService.checkDeptDataScope(deptId);
        SysDept dept = deptService.selectDeptById(deptId);
        dept.setParentName(deptService.selectDeptById(dept.getParentId()).getDeptName());
        return success(dept);
    }

    /**
     * 新增部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:add')")
    @Log(title = "部门管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysDept dept) {
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept))) {
            return error("新增部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        }
        LambdaQueryWrapper<SysDept> wq = new LambdaQueryWrapper<SysDept>();
        if (ObjectUtils.isNotEmpty(dept.getDeptCode())) {
            wq.eq(SysDept::getDeptCode, dept.getDeptCode());
        }
        SysDept sysDept = deptService.getOne(wq);
        if (ObjectUtils.isNotEmpty(sysDept)) {
            return error("新增部门'" + dept.getDeptCode() + "'失败，部门编码已存在");
        }
        dept.setCreateBy(getUserId());
        dept.setUpdateBy(getUserId());
        return toAjax(deptService.insertDept(dept));
    }

    /**
     * 修改部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:edit')")
    @Log(title = "部门管理", businessType = BusinessType.UPDATE)
    @RequestMapping("/edit")
    public AjaxResult edit(@Validated @RequestBody SysDept dept) {
        Long deptId = dept.getDeptId();
        //deptService.checkDeptDataScope(deptId);
        if (UserConstants.NOT_UNIQUE.equals(deptService.checkDeptNameUnique(dept))) {
            return error("修改部门'" + dept.getDeptName() + "'失败，部门名称已存在");
        } else if (dept.getParentId().equals(deptId)) {
            return error("修改部门'" + dept.getDeptName() + "'失败，上级部门不能是自己");
        } else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus()) && deptService.selectNormalChildrenDeptById(deptId) > 0) {
            return error("该部门包含未停用的子部门！");
        }
        LambdaQueryWrapper<SysDept> wq = new LambdaQueryWrapper<SysDept>();
        if (ObjectUtils.isNotEmpty(dept.getDeptCode())) {
            wq.eq(SysDept::getDeptCode, dept.getDeptCode());
            wq.ne(SysDept::getDeptId, dept.getDeptId());
        }
        SysDept sysDept = deptService.getOne(wq);
        if (ObjectUtils.isNotEmpty(sysDept)) {
            return error("修改部门'" + dept.getDeptCode() + "'失败，部门编码已存在");
        }
        dept.setUpdateBy(getUserId());
        return toAjax(deptService.updateDept(dept));
    }

    /**
     * 删除部门
     */
    @PreAuthorize("@ss.hasPermi('system:dept:remove')")
    @Log(title = "部门管理", businessType = BusinessType.DELETE)
    @RequestMapping("/remove/{deptId}")
    public AjaxResult remove(@PathVariable Long deptId) {
        if (deptService.hasChildByDeptId(deptId)) {
            return warn("存在下级部门,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId)) {
            return warn("部门存在用户,不允许删除");
        }
        //deptService.checkDeptDataScope(deptId);
        return toAjax(deptService.deleteDeptById(deptId));
    }

    @Log(title = "部门导出", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:role:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, @RequestBody SysDept dept) {
        List<SysDept> depts = deptService.selectDeptListAll(dept);
        if (!ObjectUtils.isEmpty(depts)) {
            for (SysDept sysDept : depts) {
                SysDept current = deptService.selectDeptById(sysDept.getParentId());
                if (!ObjectUtils.isEmpty(current)) {
                    sysDept.setParentName(current.getDeptName());
                }
            }
        }
        ExcelUtil<SysDept> util = new ExcelUtil<SysDept>(SysDept.class);
        util.exportExcel(response, depts, "部门数据");
    }

    /**
     * 遍历列表，给字段赋值
     */
    private List<SysDept> forDeptList(SysDept dept) {
        List<SysDept> list = deptService.selectDeptListAll(dept);
        for (SysDept sysDept : list) {
            for (SysDept dept1 : list) {
                if (dept1.getDeptId().equals(sysDept.getParentId())) {
                    sysDept.setParentName(dept1.getDeptName());
                    break;
                }
            }
        }
        for (SysDept sysDept : list) {
            if (ObjectUtils.isEmpty(sysDept.getParentName())) {
                sysDept.setParentName(sysDept.getDeptName());
            }
        }
        return list;
    }
}
