package com.ozs.web.controller.system;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotEmpty;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ozs.common.core.domain.entity.SysMenu;
import com.ozs.common.core.domain.vo.SysDeptVo;
import com.ozs.common.core.domain.vo.SysRoleRequestVo;
import com.ozs.common.core.domain.vo.SysUserVo;
import com.ozs.common.exception.ServiceException;
import com.ozs.service.entity.BaseRailwayManagement;
import com.ozs.service.entity.BaseUser;
import com.ozs.service.service.BaseUserService;
import com.ozs.system.domain.SysRoleMenu;
import com.ozs.system.service.DataScoreUtil;
import com.ozs.system.service.ISysMenuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.annotation.Log;
import com.ozs.common.constant.UserConstants;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.AjaxResult;
import com.ozs.common.core.domain.entity.SysDept;
import com.ozs.common.core.domain.entity.SysRole;
import com.ozs.common.core.domain.entity.SysUser;
import com.ozs.common.core.domain.model.LoginUser;
import com.ozs.common.core.page.TableDataInfo;
import com.ozs.common.enums.BusinessType;
import com.ozs.common.utils.StringUtils;
import com.ozs.common.utils.poi.ExcelUtil;
import com.ozs.framework.web.service.SysPermissionService;
import com.ozs.framework.web.service.TokenService;
import com.ozs.system.domain.SysUserRole;
import com.ozs.system.service.ISysDeptService;
import com.ozs.system.service.ISysRoleService;
import com.ozs.system.service.ISysUserService;

/**
 * 角色信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/role")
public class SysRoleController extends BaseController {
    @Autowired
    private ISysRoleService roleService;

    @Autowired
    private TokenService tokenService;

    @Autowired
    private SysPermissionService permissionService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysDeptService deptService;
    @Autowired
    private DataScoreUtil dataScoreUtil;
    @Autowired
    private ISysMenuService menuService;
    @Resource
    private BaseUserService baseUserService;

//    @PreAuthorize("@ss.hasPermi('system:role:list')")
//    @GetMapping("/list")
//    public TableDataInfo list(SysRole role)
//    {
//        startPage();
//        List<SysRole> list = roleService.selectRoleList(role);
//        return getDataTable(list);
//    }

    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @PostMapping("/list")
    @Log(title = "角色管理", businessType = BusinessType.SELECT)
    public AjaxResult page(@NotEmpty(message = "数据为空")
                           @RequestBody SysRoleRequestVo vo) {
        LambdaQueryWrapper<SysRole> lw = new LambdaQueryWrapper<SysRole>();
        if (!StringUtils.isBlank(vo.getRoleName())) {
            lw.like(SysRole::getRoleName, "%" + vo.getRoleName() + "%");
        }
        if (!ObjectUtils.isEmpty(vo.getStatus())) {
            lw.eq(SysRole::getStatus, vo.getStatus());
        }
        lw.eq(SysRole::getDelFlag,1);
        IPage<SysRole> page = roleService.page(new Page<SysRole>(vo.getPageNum(), vo.getPageSize()), lw);
        return success(page);
    }

    @Log(title = "角色管理", businessType = BusinessType.EXPORT)
    @PreAuthorize("@ss.hasPermi('system:role:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, SysRole role) {
        List<SysRole> list = roleService.selectRoleList(role);
        ExcelUtil<SysRole> util = new ExcelUtil<SysRole>(SysRole.class);
        util.exportExcel(response, list, "角色数据");
    }

    /**
     * 根据角色编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping(value = "/{roleId}")
    @Log(title = "角色管理", businessType = BusinessType.MESSAGE)
    public AjaxResult getInfo(@PathVariable Long roleId) {
//        roleService.checkRoleDataScope(roleId);
        SysRole sysRole = roleService.selectRoleById(roleId);
        if (sysRole.getRoleKey().equals(("admin"))) {
            List<SysMenu> list = menuService.list();
            List<Long> collect = list.stream().map(SysMenu::getMenuId).collect(Collectors.toList());
            Long[] strArr = collect.toArray(new Long[collect.size()]);
            sysRole.setMenuIds(strArr);
        } else {
            List<Long> rolePerms = menuService.selectMenu(sysRole.getRoleId());
            Long[] strArr = rolePerms.toArray(new Long[rolePerms.size()]);
            sysRole.setMenuIds(strArr);
        }
        return success(sysRole);
    }

    /**
     * 新增角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:add')")
    @Log(title = "角色管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysRole role) {
        if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleNameUnique(role))) {
            return error("新增角色'" + role.getRoleName() + "'失败，角色名称已存在");
        } else if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleKeyUnique(role))) {
            return error("新增角色'" + role.getRoleName() + "'失败，角色权限已存在");
        }
        role.setCreateBy(getUserId());
        role.setUpdateBy(getUserId());
        return toAjax(roleService.insertRole(role));

    }

    /**
     * 修改保存角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @RequestMapping("/edit")
    public AjaxResult edit(@Validated @RequestBody SysRole role) {
//        roleService.checkRoleAllowed(role);
//        roleService.checkRoleDataScope(role.getRoleId());
        String roleKey = roleService.getRoleKey(role.getRoleId());
        if ("admin".equals(roleKey)) {
            if (!ObjectUtils.isEmpty(role.getRoleKey())) {
                role.setRoleKey("admin");
            }
            if (!ObjectUtils.isEmpty(role.getStatus())) {
                role.setStatus("1");
            }
            return error("超级管理员角色不能修改");
        } else {
            if ("2".equals(role.getStatus())) {
                if (roleService.countUserRoleByRoleId(role.getRoleId()) > 0) {
                    throw new ServiceException(String.format("%1$s已分配,不能禁用", role.getRoleName()));
                }
            }
            if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleKeyUnique(role))) {
                return error("修改角色'" + role.getRoleName() + "'失败，角色权限已存在");
            }
        }
        if (UserConstants.NOT_UNIQUE.equals(roleService.checkRoleNameUnique(role))) {
            return error("修改角色'" + role.getRoleName() + "'失败，角色名称已存在");
        }
        role.setUpdateBy(getUserId());

        if (roleService.updateRole(role) > 0) {
            // 更新缓存用户权限
            LoginUser loginUser = getLoginUser();
            if (StringUtils.isNotNull(loginUser.getUser()) && !loginUser.getUser().isAdmin()) {
                loginUser.setPermissions(permissionService.getMenuPermission(loginUser.getUser()));
                loginUser.setUser(userService.selectUserByUserName(loginUser.getUser().getUserName()));
                tokenService.setLoginUser(loginUser);
            }
            return success();
        }
        return error("修改角色'" + role.getRoleName() + "'失败，请联系管理员");
    }

    /**
     * 修改保存数据权限
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @RequestMapping("/dataScope")
    public AjaxResult dataScope(@RequestBody SysRole role) {
        roleService.checkRoleAllowed(role);
        roleService.checkRoleDataScope(role.getRoleId());
        return toAjax(roleService.authDataScope(role));
    }

    /**
     * 状态修改
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.UPDATE)
    @RequestMapping("/changeStatus")
    public AjaxResult changeStatus(@RequestBody SysRole role) {
        roleService.checkRoleAllowed(role);
        roleService.checkRoleDataScope(role.getRoleId());
        role.setUpdateBy(getUsername());
        return toAjax(roleService.updateRoleStatus(role));
    }

    /**
     * 删除角色
     */
    @PreAuthorize("@ss.hasPermi('system:role:remove')")
    @Log(title = "角色管理", businessType = BusinessType.DELETE)
    @RequestMapping("/remove/{roleIds}")
    public AjaxResult remove(@PathVariable Long[] roleIds) {
        return toAjax(roleService.deleteRoleByIds(roleIds));
    }

    /**
     * 获取角色选择框列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping("/optionselect")
    @Log(title = "角色管理", businessType = BusinessType.OTHER)
    public AjaxResult optionselect() {
        return success(roleService.selectRoleAll());
    }

    /**
     * 查询已分配用户角色列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @GetMapping("/authUser/allocatedList")
    @Log(title = "角色管理", businessType = BusinessType.SELECT)
    public TableDataInfo allocatedList(SysUser user) {
        startPage();
        List<SysUser> list = userService.selectAllocatedList(user);
        return getDataTable(list);
    }

    /**
     * 查询未分配用户角色列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:list')")
    @GetMapping("/authUser/unallocatedList")
    @Log(title = "角色管理", businessType = BusinessType.SELECT)
    public TableDataInfo unallocatedList(SysUser user) {
        startPage();
        List<SysUser> list = userService.selectUnallocatedList(user);
        return getDataTable(list);
    }

    /**
     * 取消授权用户
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @RequestMapping("/authUser/cancel")
    public AjaxResult cancelAuthUser(@RequestBody SysUserRole userRole) {
        return toAjax(roleService.deleteAuthUser(userRole));
    }

    /**
     * 批量取消授权用户
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @RequestMapping("/authUser/cancelAll")
    public AjaxResult cancelAuthUserAll(Long roleId,@RequestBody Long[] userIds) {
        return toAjax(roleService.deleteAuthUsers(roleId, userIds));
    }

    /**
     * 批量选择用户授权
     */
    @PreAuthorize("@ss.hasPermi('system:role:edit')")
    @Log(title = "角色管理", businessType = BusinessType.GRANT)
    @RequestMapping("/authUser/selectAll")
    public AjaxResult selectAuthUserAll(Long roleId, Long[] userIds) {
        roleService.checkRoleDataScope(roleId);
        return toAjax(roleService.insertAuthUsers(roleId, userIds));
    }

    /**
     * 获取对应角色部门树列表
     */
    @PreAuthorize("@ss.hasPermi('system:role:query')")
    @GetMapping(value = "/deptTree/{roleId}")
    @Log(title = "角色管理", businessType = BusinessType.OTHER)
    public AjaxResult deptTree(@PathVariable("roleId") Long roleId) {
        AjaxResult ajax = AjaxResult.success();
        ajax.put("checkedKeys", deptService.selectDeptListByRoleId(roleId));
        ajax.put("depts", deptService.selectDeptTreeList(new SysDeptVo()));
        return ajax;
    }
}
