package com.ozs.web.controller.tool;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import com.ozs.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ozs.common.core.controller.BaseController;
import com.ozs.common.core.domain.R;
import com.ozs.common.utils.StringUtils;

/**
 *  用户测试方法
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/test")
public class TestController extends BaseController {

    @Autowired
    private com.ozs.web.core.util.CameraUtil cameraUtil;

    @Autowired
    private com.ozs.web.core.util.CmdCameraUtil cmdCameraUtil;
    private final static Map<Integer, UserEntity> users = new LinkedHashMap<Integer, UserEntity>();

    {
        users.put(1, new UserEntity(1, "admin", "admin123", "15888888888"));
        users.put(2, new UserEntity(2, "ry", "admin123", "15666666666"));
    }

    @GetMapping("/deleteFlv")
    public R<String> deleteFlv() {
        try {
            cameraUtil.deleteFlv();
        } catch (Exception e) {
            return R.ok(e.getMessage().toString());
        }
        return R.ok();
    }
    @GetMapping("/deleteTmpFlv")
    public R<String> deleteTmpFlv() {
        try {
            cameraUtil.deleteTmpFlv("1");
        } catch (Exception e) {
            return R.ok(e.getMessage().toString());
        }
        return R.ok();
    }

    @GetMapping("/restart")
    public R<String> restart() {
        try {
            cameraUtil.restart();
        } catch (Exception e) {
            return R.ok(e.getMessage().toString());
        }
        return R.ok();
    }

    @GetMapping("/cmd")
    public R<String> cmd(String cmd) {

        cmdCameraUtil.cmd(cmd);
        return R.ok();
    }
    @GetMapping("/mergeVideo")
    public R<String> mergeVideo() {
        try {
            cameraUtil.mergeVideo();
        } catch (Exception e) {
            return R.ok(e.getMessage().toString());
        }
        return R.ok();
    }

    @GetMapping("/deleteTs")
    public R<String> deleteTs() {
        try {
            cameraUtil.deleteTsFile();
        } catch (Exception e) {
            return R.ok(e.getMessage().toString());
        }
        return R.ok();
    }

    @GetMapping("/list")
    public R<List<UserEntity>> userList() {
        List<UserEntity> userList = new ArrayList<UserEntity>(users.values());
        return R.ok(userList);
    }

    @GetMapping("/{userId}")
    public R<UserEntity> getUser(@PathVariable Integer userId) {
        if (!users.isEmpty() && users.containsKey(userId)) {
            return R.ok(users.get(userId));
        } else {
            return R.fail("用户不存在");
        }
    }

    
    @PostMapping("/save")
    public R<String> save(UserEntity user) {
        if (StringUtils.isNull(user) || StringUtils.isNull(user.getUserId())) {
            return R.fail("用户ID不能为空");
        }
        users.put(user.getUserId(), user);
        return R.ok();
    }

    @RequestMapping("/update")
    public R<String> update(@RequestBody UserEntity user) {
        if (StringUtils.isNull(user) || StringUtils.isNull(user.getUserId())) {
            return R.fail("用户ID不能为空");
        }
        if (users.isEmpty() || !users.containsKey(user.getUserId())) {
            return R.fail("用户不存在");
        }
        users.remove(user.getUserId());
        users.put(user.getUserId(), user);
        return R.ok();
    }

    @RequestMapping("/{userId}")
    public R<String> delete(@PathVariable Integer userId) {
        if (!users.isEmpty() && users.containsKey(userId)) {
            users.remove(userId);
            return R.ok();
        } else {
            return R.fail("用户不存在");
        }
    }
}

class UserEntity {
    private Integer userId;

    private String username;

    private String password;

    private String mobile;

    public UserEntity() {

    }

    public UserEntity(Integer userId, String username, String password, String mobile) {
        this.userId = userId;
        this.username = username;
        this.password = password;
        this.mobile = mobile;
    }

    public Integer getUserId() {
        return userId;
    }

    public void setUserId(Integer userId) {
        this.userId = userId;
    }

    public String getUsername() {
        return username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getMobile() {
        return mobile;
    }

    public void setMobile(String mobile) {
        this.mobile = mobile;
    }

    public static void main(String[] args) {
        System.out.println(SecurityUtils.encryptPassword("yn5aq5Mt.106.tky"));
    }
}
